@@ -31,7 +31,7 @@
 
 #define exit(_a_) _exit(_a_)
 
-static const char rcsid[]="$Id: lockdaemon.c,v 1.7 2003/01/05 04:01:16 mrsam Exp $";
+static const char rcsid[]="$Id: lockdaemon.c,v 1.8 2005/07/02 17:22:13 mrsam Exp $";
 
 static int start1(const char *, int);
 
@@ -134,7 +134,17 @@ int	lockfd;
 
 	/* Attempt to obtain a lock */
 
-	if ((lockfd=open(lockfile, O_RDWR|O_CREAT, 0600)) < 0)
+	lockfd=open(lockfile, O_RDWR|O_CREAT, 0600);
+
+	if (lockfd < 0)
+	{
+		/* Perhaps an upgraded daemon runs under new uid? */
+
+		unlink(lockfile);
+		lockfd=open(lockfile, O_RDWR|O_CREAT, 0600);
+	}
+
+	if (lockfd < 0)
 	{
 		perror(lockfile);
 		exit(1);
@@ -190,6 +200,7 @@ char	*p=strcat(libmail_str_pid_t(getpid(), buf), "\n");
 size_t	l=strlen(p);
 FILE	*fp;
 
+	unlink(pidfile); 
 	if ((fp=fopen(pidfile, "w")) == NULL ||
 		fprintf(fp, "%s", p) < 0 || fflush(fp) < 0 || fclose(fp))
 	{
