@@ -1,5 +1,5 @@
 /*
-** Copyright 2000-2003 Double Precision, Inc.
+** Copyright 2000-2005 Double Precision, Inc.
 ** See COPYING for distribution information.
 */
 
@@ -17,6 +17,17 @@
 #include	<ctype.h>
 #include	<errno.h>
 #include	<sys/types.h>
+#include	"maildirmisc.h"
+
+#if	HAVE_PCRE_H
+#include	<pcre.h>
+#else
+#if	HAVE_PCRE_PCRE_H
+#include	<pcre/pcre.h>
+#define HAVE_PCRE_H	1
+#endif
+#endif
+
 #if	HAVE_SYS_STAT_H
 #include	<sys/stat.h>
 #endif
@@ -24,7 +35,7 @@
 #include	<unistd.h>
 #endif
 
-static const char rcsid[]="$Id: maildirfilter.c,v 1.20 2003/04/20 03:44:42 mrsam Exp $";
+static const char rcsid[]="$Id: maildirfilter.c,v 1.24 2005/09/24 00:27:05 mrsam Exp $";
 
 struct maildirfilterrule *maildir_filter_appendrule(struct maildirfilter *r,
 					const char *name,
@@ -157,11 +168,12 @@ const char *c;
 	}
 	else if (!(flags & MFR_PLAINSTRING))
 	{
-/*
-** This is ugly.  We: make sure parenthesis match up, and look out for
-** special characters.
-*/
-	int lparencount=0;
+		/*
+		** Let PCRE decide if this is a valid pattern.
+		**
+		** One exception: the forward slash character, and some other
+		** special characters, must always be escaped.
+		*/
 
 		while (*c)
 		{
@@ -179,7 +191,6 @@ const char *c;
 			if (*c == '(')
 			{
 				if (type == hasrecipient)	return (-1);
-				++lparencount;
 				++c;
 				if (*c == ')')	return (-1);
 				continue;
@@ -187,9 +198,7 @@ const char *c;
 			if (*c == ')')
 			{
 				if (type == hasrecipient)	return (-1);
-				if (lparencount == 0)	return (-1);
 				++c;
-				--lparencount;
 				continue;
 			}
 			if (*c == '[')	/* This is a set */
@@ -220,6 +229,7 @@ const char *c;
 						return (-1);
 					if (!*c)	return (-1);
 					++c;
+					if (*c == ']')	break;
 				}
 				++c;
 				continue;
@@ -233,7 +243,31 @@ const char *c;
 			if (!*c)	return (-1);
 			++c;
 		}
-		if (lparencount)	return (-1);	/* not balanced */
+
+#if HAVE_PCRE_H
+		switch (type) {
+		case contains:
+		case startswith:
+		case endswith:
+			{
+				const char *errptr;
+				int errindex;
+
+				pcre *p=pcre_compile(value, 0,
+						     &errptr,
+						     &errindex,
+						     0);
+
+
+				if (p == NULL)
+					return -1;
+				pcre_free(p);
+			}
+			break;
+		default:
+			break;
+		}
+#endif
 	}
 
 	/* validate FROM header */
@@ -277,16 +311,19 @@ const char *c;
 
 		maildir_filter_autoresp_info_free(&ai);
 	}
-	else if (strcmp(c, "."))
+	else
 	{
-		for ( ; *c; c++)
-		{
-			if (*c == '.' && (c[1] == '.' || c[1] == 0))
-				return (-1);
-			if ((int)(unsigned char)*c < ' ' || *c == '\\'
-				|| *c == '|' || *c == '!')
-				return (-1);
-		}
+		char *s;
+
+		if (strcmp(c, INBOX) &&
+		    strncmp(c, INBOX ".", sizeof(INBOX)))
+			return -1;
+
+		s=maildir_name2dir(".", c);
+
+		if (!s)
+			return -1;
+		free(s);
 	}
 
 	/* OK, we're good */
@@ -301,13 +338,8 @@ const char *c;
 	if (p->fieldvalue)	free(p->fieldvalue);
 	if ((p->fieldvalue=strdup(value ? value:"")) == 0)	return (-1);
 	if (p->tofolder)	free(p->tofolder);
-	if ((p->tofolder=malloc(strlen(folder)+2)) == 0)	return (-1);
-	*p->tofolder=0;
-	/* Prefix with dot by default */
-	if (*folder != '*' && *folder != '!' && *folder != '.'
-	    && *folder != '+')
-		strcpy(p->tofolder, ".");
-	strcat(p->tofolder, folder);
+	if ((p->tofolder=malloc(strlen(folder)+1)) == 0)	return (-1);
+	strcpy(p->tofolder, folder);
 
 	if (p->fromhdr)		free(p->fromhdr);
 	if ((p->fromhdr=strdup(fromhdr ? fromhdr:"")) == NULL)
@@ -417,7 +449,10 @@ struct maildirfilterrule *p;
 			typelist[p->type].name);
 		fprintf(f, "##Header:%s\n", fieldname);
 		fprintf(f, "##Value:%s\n", fieldvalue);
-		fprintf(f, "##Folder:%s\n", tofolder);
+		fprintf(f, "##Folder:%s\n",
+			strcmp(tofolder, INBOX) == 0 ? ".":
+			strncmp(tofolder, INBOX ".", sizeof(INBOX)) == 0
+			? strchr(tofolder, '.'):tofolder);
 		fprintf(f, "##From:%s\n", p->fromhdr ? p->fromhdr:"");
 
 		if (p->flags & MFR_PLAINSTRING)
@@ -542,15 +577,20 @@ struct maildirfilterrule *p;
 		}
 		else
 		{
-			if (*tofolder == '.')	++tofolder;
-				/* Strip leading dot */
+			char *s;
 
-			fprintf(f,
-				"   %s \"%s%s%s/.\"\n",
-				p->flags & MFR_CONTINUE ? "cc":"to",
-				maildirpath,
-				*tofolder ? "/.":"",
-				tofolder);
+			s=maildir_name2dir(maildirpath, tofolder);
+
+			if (!s)
+				fprintf(f, "  # INTERNAL ERROR in maildir_name2dir\n");
+			else
+			{
+				fprintf(f,
+					"   %s \"%s/.\"\n",
+					p->flags & MFR_CONTINUE ? "cc":"to",
+					s);
+				free(s);
+			}
 		}
 		fprintf(f, "}\n\n");
 	}
@@ -660,7 +700,17 @@ int	flags;
 		if (strncasecmp(p, "Folder:", 7) == 0)
 		{
 			p += 7;
-			SET(new_folder, p);
+
+			if (*p == '.')
+			{
+				strcpy(new_folder, INBOX);
+			}
+			else
+				new_folder[0]=0;
+
+			if (strcmp(p, "."))
+				strncat(new_folder, p,
+					sizeof(new_folder)-1-strlen(new_folder));
 			continue;
 		}
 
@@ -691,17 +741,12 @@ int	flags;
 		if (strncasecmp(p, "Name:", 5) == 0)
 		{
 		int dummy;
-		struct maildirfilterrule *rr;
 
 			p += 5;
-			if ((rr=maildir_filter_appendrule(r, p, new_type, flags,
-					      new_header,
-					      new_value, new_folder,
-					      new_autoreplyfrom, &dummy)) == 0)
-			{
-				fclose(f);
-				return (MF_LOADERROR);
-			}
+			maildir_filter_appendrule(r, p, new_type, flags,
+						  new_header,
+						  new_value, new_folder,
+						  new_autoreplyfrom, &dummy);
 			new_type=contains;
 			new_header[0]=0;
 			new_value[0]=0;
