@@ -1,5 +1,5 @@
 /*
-** Copyright 1998 - 2003 Double Precision, Inc.
+** Copyright 1998 - 2006 Double Precision, Inc.
 ** See COPYING for distribution information.
 */
 
@@ -45,12 +45,15 @@
 #include	<unistd.h>
 #endif
 #include	<time.h>
+#include	<ctype.h>
 #include	<numlib/numlib.h>
 
-static const char rcsid[]="$Id: maildirquota.c,v 1.21 2003/02/13 03:19:45 mrsam Exp $";
+static const char rcsid[]="$Id: maildirquota.c,v 1.29 2006/02/08 02:25:07 mrsam Exp $";
 
 static void parsequotastr(const char *, struct maildirquota *);
 
+#define DIG(c) ( (c) >= '0' && (c) <= '9')
+
 /* Read the maildirsize file */
 
 static int maildir_openquotafile_init(struct maildirsize *info,
@@ -194,10 +197,11 @@ static int do_maildir_openquotafile(struct maildirsize *info,
 	first=1;
 	while (*p)
 	{
-		long n=0;
+		off_t n=0;
 		int c=0;
 		char	*q=p;
-	
+		int	neg;
+
 		while (*p)
 			if (*p++ == '\n')
 			{
@@ -211,30 +215,76 @@ static int do_maildir_openquotafile(struct maildirsize *info,
 			first=0;
 			continue;
 		}
-		sscanf(q, "%ld %d", &n, &c);
-		info->size.nbytes += n;
-		info->size.nmessages += c;
+
+		while (*q && isspace((int)(unsigned char)*q))
+			++q;
+
+		neg=0;
+		if (*q == '-')
+		{
+			neg=1;
+			++q;
+		}
+
+		if (DIG(*q))
+		{
+			while (DIG(*q))
+			{
+				n=n*10 + (*q++ - '0');
+			}
+
+			if (neg)
+				n= -n;
+
+			neg=0;
+			while (*q && isspace((int)(unsigned char)*q))
+				++q;
+
+			if (*q == '-')
+			{
+				neg=1;
+				++q;
+			}
+			if (DIG(*q))
+			{
+				while (DIG(*q))
+				{
+					c=c*10 + (*q++ - '0');
+				}
+
+				if (neg)
+					c= -c;
+
+				info->size.nbytes += n;
+				info->size.nmessages += c;
+			}
+		}
+
 		++ info->nlines;
 	}
+	if (info->size.nbytes < 0 ||
+	    info->size.nmessages < 0)
+		info->recalculation_needed=1;
 	return (0);
 }
 
 static void parsequotastr(const char *quota, struct maildirquota *q)
 {
 	off_t i;
+	const char *quota_start = quota;
 
 	q->nbytes=0;
 	q->nmessages=0;
 
 	while (quota && *quota)
 	{
-		if (*quota < '0' || *quota > '9')
+		if (!DIG(*quota))
 		{
 			++quota;
 			continue;
 		}
 		i=0;
-		while (*quota >= '0' && *quota <= '9')
+		while (DIG(*quota))
 			i=i*10 + (*quota++ - '0');
 		switch (*quota)	{
 		case MDQUOTA_SIZE:
@@ -243,6 +293,10 @@ static void parsequotastr(const char *quota, struct maildirquota *q)
 		case MDQUOTA_COUNT:
 			q->nmessages=i;
 			break;
+		default:
+			fprintf(stderr, "WARN: quota string '%s' not parseable\n",
+				quota_start);
+			break;
 		}
 	}
 }
@@ -323,14 +377,14 @@ static int countsubdir(const char *, const char *,
 static int statcurnew(const char *, time_t *);
 static int statsubdir(const char *, const char *, time_t *);
 
-static int	doaddquota(struct maildirsize *, int, long, int, int);
+static int	doaddquota(struct maildirsize *, int, off_t, int, int);
 
 static int docheckquota(struct maildirsize *info,
-			long xtra_size,
+			off_t xtra_size,
 			int xtra_cnt, int *percentage);
 
 int maildir_checkquota(struct maildirsize *info,
-		       long xtra_size,
+		       off_t xtra_size,
 		       int xtra_cnt)
 {
 	int	dummy;
@@ -347,7 +401,7 @@ int maildir_readquota(struct maildirsize *info)
 }
 
 static int docheckquota(struct maildirsize *info,
-			long xtra_size,
+			off_t xtra_size,
 			int xtra_cnt,
 			int *percentage)
 {
@@ -434,8 +488,8 @@ static int docheckquota(struct maildirsize *info,
 	if (doaddquota(info, maildirsize_fd, maildirsize_size,
 		maildirsize_cnt, 1))
 	{
-		free(newmaildirsizename);
 		unlink(newmaildirsizename);
+		free(newmaildirsizename);
 		close(maildirsize_fd);
 		errno=EIO;
 		return (-1);
@@ -506,7 +560,7 @@ static int docheckquota(struct maildirsize *info,
 }
 
 int	maildir_addquota(struct maildirsize *info,
-			 long maildirsize_size, int maildirsize_cnt)
+			 off_t maildirsize_size, int maildirsize_cnt)
 {
 	if (info->fd < 0)
 		return (0);
@@ -516,7 +570,7 @@ int	maildir_addquota(struct maildirsize *info,
 }
 
 static int doaddquota(struct maildirsize *info, int maildirsize_fd,
-		      long maildirsize_size, int maildirsize_cnt,
+		      off_t maildirsize_size, int maildirsize_cnt,
 		      int isnew)
 {
 	char	n[NUMBUFSIZE];
@@ -557,10 +611,10 @@ static int doaddquota(struct maildirsize *info, int maildirsize_fd,
 	}
 
 	sprintf(buf + strlen(buf),
-		"%8s ", libmail_str_off_t(maildirsize_size, n));
+		"%12s ", libmail_str_off_t(maildirsize_size, n));
 
 	sprintf(buf + strlen(buf),
-		"%4s\n", libmail_str_off_t(maildirsize_cnt, n));
+		"%12s\n", libmail_str_off_t(maildirsize_cnt, n));
 
 	p=buf;
 	cnt=strlen(buf);
@@ -637,7 +691,11 @@ char	*p;
 int	n;
 
 	if ( *subdir != '.' || strcmp(subdir, ".") == 0 ||
-		strcmp(subdir, "..") == 0 || strcmp(subdir, "." TRASH) == 0)
+		strcmp(subdir, "..") == 0
+#ifndef TRASHQUOTA
+				|| strcmp(subdir, "." TRASH) == 0
+#endif
+		)
 		return (0);
 
 	p=(char *)malloc(strlen(dir)+strlen(subdir)+2);
@@ -694,7 +752,7 @@ struct	stat	stat_buf;
 char	*p;
 DIR	*dirp;
 struct dirent *de;
-unsigned long	s;
+unsigned long s;
 
 	if (stat(dir, &stat_buf))	return (0);	/* Ignore */
 	if (stat_buf.st_mtime > *dirstamp)	*dirstamp=stat_buf.st_mtime;
@@ -792,7 +850,7 @@ int maildirquota_countfile(const char *n)
 
 int maildir_quota_add_start(const char *maildir,
 			    struct maildirsize *info,
-			    long msgsize, int nmsgs,
+			    off_t msgsize, int nmsgs,
 			    const char *newquota)
 {
 	struct maildirquota mq;
@@ -847,14 +905,14 @@ int maildir_quota_add_start(const char *maildir,
 }
 
 void maildir_quota_add_end(struct maildirsize *info,
-			   long msgsize, int nmsgs)
+			   off_t msgsize, int nmsgs)
 {
 	maildir_addquota(info, msgsize, nmsgs);
 	maildir_closequotafile(info);
 }
 
 void maildir_quota_deleted(const char *maildir,
-			   long nbytes,	/* Must be negative */
+			   off_t nbytes,	/* Must be negative */
 			   int nmsgs)	/* Must be negative */
 {
 	struct maildirsize info;
@@ -881,7 +939,7 @@ void maildir_quota_recalculate(const char *maildir)
 
 int maildir_quota_delundel_start(const char *maildir,
 				 struct maildirsize *info,
-				 long msgsize, int nmsgs)
+				 off_t msgsize, int nmsgs)
 {
 #if TRASHQUOTA
 	return (0);
@@ -897,7 +955,7 @@ int maildir_quota_delundel_start(const char *maildir,
 }
 
 void maildir_quota_delundel_end(struct maildirsize *info,
-				long msgsize, int nmsgs)
+				off_t msgsize, int nmsgs)
 {
 #if TRASHQUOTA
 	return;
@@ -924,7 +982,7 @@ void maildir_quota_set(const char *dir, const char *quota)
 
 static void do_deliver_warning(const char *msgfile, const char *dir)
 {
-int	fdin, fd;
+int	fdin, fd, ret;
 FILE *fpout;
 time_t	t;
 size_t	l, msg_len;
@@ -950,7 +1008,9 @@ struct maildir_tmpcreate_info createInfo;
 
 	strcat(strcpy(qname, dir), "/quotawarn");
 	time(&t);
-	if (stat(qname, &sb) == 0 && ((sb.st_mtime + 86400) > t))
+	ret = stat(qname, &sb);
+	if ((ret == -1 && errno != ENOENT) ||
+	    (ret == 0 && (sb.st_mtime + 86400) > t))
 	{
 		free(qname);
 		close(fdin);
@@ -959,12 +1019,12 @@ struct maildir_tmpcreate_info createInfo;
 
 	fd = open(qname, O_WRONLY|O_CREAT|O_TRUNC, 0644);
 	free(qname);
-	if (!fd)
+	if (fd == -1)
 	{
 		close(fdin);
 		return;
 	}
-	write(fd, buf, 0);
+	write(fd, buf, 1);
 	close(fd);
 
 	strcpy(buf, "Date: ");
@@ -974,8 +1034,8 @@ struct maildir_tmpcreate_info createInfo;
 	hostname[0]=0;
 	hostname[sizeof(hostname)-1]=0;
 	gethostname(hostname, sizeof(hostname)-1);
-	sprintf(buf, "%sMessage-Id: <%lu.overquota@%-1.256s>\n",
-		buf, (unsigned long)t, hostname);
+	sprintf(buf+strlen(buf), "Message-Id: <%lu.overquota@%-1.256s>\n",
+		(unsigned long)t, hostname);
 
 	if (stat(msgfile, &sb) < 0) {
 		close(fdin);
