@@ -41,7 +41,7 @@
 #include	"maildircreate.h"
 #include	"maildirsharedrc.h"
 
-static const char rcsid[]="$Id: maildirshared.c,v 1.19 2003/01/20 13:47:20 mrsam Exp $";
+static const char rcsid[]="$Id: maildirshared.c,v 1.21 2005/06/19 13:15:28 mrsam Exp $";
 
 /* Prerequisited for shared folder support */
 
@@ -316,7 +316,7 @@ char	*s;
 	s=maildir_shareddir(maildir, folder);
 	if (!s)	return (-1);
 
-	if (maildir_mddelete(s))
+	if (maildir_del(s))
 	{
 		free(s);
 		return (-1);
@@ -327,44 +327,6 @@ char	*s;
 	return (0);
 }
 
-char *maildir_shareddir(const char *maildir, const char *sharedname)
-{
-char    *p, *q;
-const char *r;
-size_t  l;
-
-	if (!maildir)   maildir=".";
-        l=strlen(maildir);
-
-	if (strchr(sharedname, '.') == 0 || *sharedname == '.' ||
-		strchr(sharedname, '/'))
-	{
-		errno=EINVAL;
-		return (0);
-	}
-
-	for (r=sharedname; *r; r++)
-	{
-		if (*r == '.' && (r[1] == '.' || r[1] == '\0'))
-		{
-			errno=EINVAL;
-			return (0);
-		}
-	}
-
-	p=malloc(strlen(maildir)+sizeof("/" SHAREDSUBDIR "/")+strlen(sharedname));
-	if (!p)	return (0);
-
-        *p=0;
-        if (strcmp(maildir, "."))
-		strcat(strcpy(p, maildir), "/");
-	strcat(p, SHAREDSUBDIR "/");
-	q=p+strlen(p);
-	strcpy(q, sharedname);
-	*strchr(q, '.')='/';
-	return (p);
-}
-
 /*                    LET'S SYNC IT                  */
 
 static void do_maildir_shared_sync(const char *, const char *);
@@ -925,11 +887,13 @@ int maildir_shared_unsubscribe(const char *maildir, const char *folder)
 	return (-1);
 }
 
+#if 0
 char *maildir_shareddir(const char *maildir, const char *sharedname)
 {
 	errno=EINVAL;
 	return (0);
 }
+#endif
 
 void maildir_shared_sync(const char *maildir)
 {
