@@ -1,16 +1,7 @@
-/* DSTART                                                                    */
-/*                                                                           */
-/*           maildrop - mail delivery agent with filtering abilities         */
-/*                                                                           */
-/*  Copyright 1998-1999, Double Precision Inc.                               */
-/*                                                                           */
-/*  This program is distributed under the terms of the GNU General Public    */
-/*  License. See COPYING for additional information.                         */
-/* DEND                                                                      */
 #ifndef	tempfile_h
 #define	tempfile_h
 
-static const char tempfile_h_rcsid[]="$Id: tempfile.h 1.3 1999/09/10 02:10:05 mrsam Exp $";
+static const char tempfile_h_rcsid[]="$Id: tempfile.h,v 1.3 1999/09/10 02:10:05 mrsam Exp $";
 
 #include	<sys/types.h>
 #include	"config.h"
