@@ -9,7 +9,7 @@
 #include	"numlib.h"
 #include	<string.h>
 
-static const char rcsid[]="$Id: strsize.c,v 1.1 2001/08/12 15:46:40 mrsam Exp $";
+static const char rcsid[]="$Id: strsize.c,v 1.2 2003/01/05 04:01:17 mrsam Exp $";
 
 static void cat_n(char *buf, unsigned long n)
 {
@@ -25,7 +25,7 @@ char    *p=bb+sizeof(bb)-1;
         strcat(buf, p);
 }
 
-char *str_sizekb(unsigned long n, char *sizebuf)
+char *libmail_str_sizekb(unsigned long n, char *sizebuf)
 {
         /* If size is less than 1K bytes, display it as 0.xK */
 
