@@ -1,5 +1,5 @@
 /*
-** Copyright 2000-2001 Double Precision, Inc.  See COPYING for
+** Copyright 2000-2004 Double Precision, Inc.  See COPYING for
 ** distribution information.
 */
 
@@ -24,6 +24,7 @@
 #include	<pwd.h>
 #include	<fcntl.h>
 #include	<signal.h>
+#include	"rfc822/encode.h"
 #include	"rfc2045.h"
 #include	"rfc2045charset.h"
 #if HAVE_UNISTD_H
@@ -39,7 +40,7 @@
 int gethostname(const char *, size_t);
 #endif
 
-static const char rcsid[]="$Id: makemime.c,v 1.11 2003/03/07 00:47:30 mrsam Exp $";
+static const char rcsid[]="$Id: makemime.c,v 1.14 2004/05/23 14:28:24 mrsam Exp $";
 
 struct arg_list {
 	struct arg_list *next;
@@ -630,10 +631,19 @@ static int encode_outfp(const char *p, size_t n, void *vp)
 	return 0;
 }
 
+static int do_printRfc2231Attr(const char *param,
+			       const char *value,
+			       void *voidArg)
+{
+	fprintf( ((struct mimestruct *)voidArg)->outputfp,
+		 ";\n  %s=%s", param, value);
+	return 0;
+}
+
 static void createsimplemime(struct mimestruct *m)
 {
 struct	arg_list *a;
-struct rfc2045_encode_info encode_info;
+struct libmail_encode_info encode_info;
 
 	/* Determine encoding by reading the file, as follows:
 	**
@@ -652,7 +662,9 @@ struct rfc2045_encode_info encode_info;
 			goodexit(m, 1);
 		}
 
-		m->mimeencoding=rfc2045_encode_autodetect_fp(m->inputfp1, 1);
+		m->mimeencoding=libmail_encode_autodetect_fppos(m->inputfp1,
+								m->textplaincharset,
+								0, -1);
 
 		if (ferror(m->inputfp1)
 			|| fseek(m->inputfp1, orig_pos, SEEK_SET)<0)
@@ -689,33 +701,15 @@ struct rfc2045_encode_info encode_info;
 	{
 		const char *chset=m->textplaincharset ? m->textplaincharset
 			: "iso-8859-1";
-		char *s;
-		const char nameStr[]="name";
-		int len;
 
-		rfc2231_attrCreate(nameStr, m->contentname, chset,
-				   NULL,
-				   &len,
-				   "");
-
-		s=malloc(len);
-
-		if (s)
-		{
-			rfc2231_attrCreate(nameStr, m->contentname, chset,
-					   s,
-					   &len,
-					   "");
-
-			fprintf(m->outputfp, "; %s", s);
-			free(s);
-		}
+		rfc2231_attrCreate("name", m->contentname, chset, NULL,
+				   do_printRfc2231Attr, m);
 	}
 
 	fprintf(m->outputfp, "\nContent-Transfer-Encoding: %s\n\n",
 		m->mimeencoding);
 
-	rfc2045_encode_start(&encode_info, m->mimeencoding,
+	libmail_encode_start(&encode_info, m->mimeencoding,
 			     &encode_outfp,
 			     &m->outputfp);
 	{
@@ -725,11 +719,11 @@ struct rfc2045_encode_info encode_info;
 		while ((n=fread(input_buf, 1, sizeof(input_buf),
 				m->inputfp1)) > 0)
 		{
-			if ( rfc2045_encode(&encode_info, input_buf, n))
+			if ( libmail_encode(&encode_info, input_buf, n))
 				break;
 		}
 
-		rfc2045_encode_end(&encode_info);
+		libmail_encode_end(&encode_info);
 	}
 }
 
