@@ -1,5 +1,5 @@
 /*
-** Copyright 2000 Double Precision, Inc.  See COPYING for
+** Copyright 2000-2004 Double Precision, Inc.  See COPYING for
 ** distribution information.
 */
 
@@ -9,13 +9,14 @@
 #include	"rfc822/rfc2047.h"
 #include	"rfc2045charset.h"
 #include	"rfc822/rfc822.h"
+#include	"unicode/unicode.h"
 #include	<stdio.h>
 #include	<unistd.h>
 #include	<stdlib.h>
 #include	<string.h>
 #include	<ctype.h>
 
-static const char rcsid[]="$Id: rfc2045reply.c,v 1.9 2003/03/07 00:47:31 mrsam Exp $";
+static const char rcsid[]="$Id: rfc2045reply.c,v 1.13 2004/09/03 01:28:47 mrsam Exp $";
 
 extern void rfc2045_enomem();
 
@@ -156,6 +157,12 @@ static int mkforward(struct rfc2045_mkreplyinfo *ri)
 	int is_flowed_fwd=0;
 
 	struct rfc2045headerinfo *hi;
+#if	HAVE_UNICODE
+	const struct unicode_info *uiptr=NULL;
+
+	if (ri->charset && *(ri->charset))
+		uiptr = unicode_find(ri->charset);
+#endif
 
 	rfc2045_mimepos(ri->rfc2045partp, &start_pos, &end_pos, &start_body,
 		&dummy, &dummy);
@@ -195,8 +202,7 @@ static int mkforward(struct rfc2045_mkreplyinfo *ri)
 	writes(ri, "Subject: ");
 	if (subject)
 	{
-		int	dummy;
-		char	*s=rfc822_coresubj_nouc(subject, &dummy);
+		char	*s=rfc822_coresubj_keepblobs(subject);
 
 		if (!s)
 			return (-1);
@@ -270,10 +276,56 @@ static int mkforward(struct rfc2045_mkreplyinfo *ri)
 
 		/* Copy original headers. */
 		
-		if (lseek(ri->fd, orig_startpos, SEEK_SET) == -1)
-			return (-1);
+		hi=rfc2045header_start(ri->fd, ri->rfc2045partp);
+		for (;;)
+		{
+			if (rfc2045header_get(hi, &header, &value,
+					      RFC2045H_NOLC|RFC2045H_KEEPNL))
+			{
+				rfc2045header_end(hi);
+				break;
+			}
+			if (!header)
+				break;
+			if (strcasecmp(header, "subject") == 0 ||
+			    strcasecmp(header, "from") == 0 ||
+			    strcasecmp(header, "to") == 0 ||
+			    strcasecmp(header, "cc") == 0 ||
+			    strcasecmp(header, "date") == 0 ||
+			    strcasecmp(header, "message-id") == 0 ||
+			    strcasecmp(header, "resent-from") == 0 ||
+			    strcasecmp(header, "resent-to") == 0 ||
+			    strcasecmp(header, "resent-cc") == 0 ||
+			    strcasecmp(header, "resent-date") == 0 ||
+			    strcasecmp(header, "resent-message-id") == 0)
+			{
+				if (subject) free(subject);
+#if	HAVE_UNICODE
+				if (uiptr)
+					subject=rfc2047_decode_unicode(value,
+						uiptr, RFC2047_DECODE_REPLACE);
+				else
+#endif
+				subject=rfc2047_decode_enhanced(value,
+								ri->charset);
+				if (subject)
+				{
+					(*ri->write_func)(header,
+							  strlen(header),
+							  ri->voidarg);
+					(*ri->write_func)(": ", 2, ri->voidarg);
+					(*ri->write_func)(subject,
+							  strlen(subject),
+							  ri->voidarg);
+					(*ri->write_func)("\n", 1, ri->voidarg);
+				}
+			}
+		}
+		rfc2045header_end(hi);
+		(*ri->write_func)("\n", 1, ri->voidarg);
 
-		forwardbody(ri, start_body - orig_startpos);
+		if (lseek(ri->fd, start_body, SEEK_SET) == -1)
+			return (-1);
 
 		start_pos=start_body;
 
@@ -287,11 +339,57 @@ static int mkforward(struct rfc2045_mkreplyinfo *ri)
 		}
 
 		/* Copy original headers. */
-		
-		if (lseek(ri->fd, orig_startpos, SEEK_SET) == -1)
-			return (-1);
 
-		forwardbody(ri, start_body - orig_startpos);
+		hi=rfc2045header_start(ri->fd, ri->rfc2045partp);
+		for (;;)
+		{
+			if (rfc2045header_get(hi, &header, &value,
+					      RFC2045H_NOLC|RFC2045H_KEEPNL))
+			{
+				rfc2045header_end(hi);
+				break;
+			}
+			if (!header)
+				break;
+			if (strcasecmp(header, "subject") == 0 ||
+			    strcasecmp(header, "from") == 0 ||
+			    strcasecmp(header, "to") == 0 ||
+			    strcasecmp(header, "cc") == 0 ||
+			    strcasecmp(header, "date") == 0 ||
+			    strcasecmp(header, "message-id") == 0 ||
+			    strcasecmp(header, "resent-from") == 0 ||
+			    strcasecmp(header, "resent-to") == 0 ||
+			    strcasecmp(header, "resent-cc") == 0 ||
+			    strcasecmp(header, "resent-date") == 0 ||
+			    strcasecmp(header, "resent-message-id") == 0)
+			{
+				if (subject) free(subject);
+#if	HAVE_UNICODE
+				if (uiptr)
+					subject=rfc2047_decode_unicode(value,
+						uiptr, RFC2047_DECODE_REPLACE);
+				else
+#endif
+				subject=rfc2047_decode_enhanced(value,
+								ri->charset);
+				if (subject)
+				{
+					(*ri->write_func)(header,
+							  strlen(header),
+							  ri->voidarg);
+					(*ri->write_func)(": ", 2, ri->voidarg);
+					(*ri->write_func)(subject,
+							  strlen(subject),
+							  ri->voidarg);
+					(*ri->write_func)("\n", 1, ri->voidarg);
+				}
+			}
+		}
+		rfc2045header_end(hi);
+		(*ri->write_func)("\n", 1, ri->voidarg);
+
+		if (lseek(ri->fd, start_body, SEEK_SET) == -1)
+			return (-1);
 
 		/* Decode the body of the MIME section */
 
@@ -337,7 +435,8 @@ static int mkforward(struct rfc2045_mkreplyinfo *ri)
 		{
 			char *p=rfc2047_encode_str(ri->forwarddescr,
 						   ri->charset ?
-						   ri->charset : "iso-8859-1");
+						   ri->charset : "iso-8859-1",
+						   rfc2047_qp_allow_any);
 
 			writes(ri, "Content-Description: ");
 			writes(ri, p ? p:"");
@@ -350,13 +449,16 @@ static int mkforward(struct rfc2045_mkreplyinfo *ri)
 	{
 		off_t save_start_body=ri->rfc2045partp->startbody;
 		off_t save_end_pos=ri->rfc2045partp->endpos;
+#if 0
 		char *save_te=
 			ri->rfc2045partp->content_transfer_encoding;
+#endif
 
 		ri->rfc2045partp->startbody=start_pos;
 		ri->rfc2045partp->endpos=end_pos;
+#if 0
 		ri->rfc2045partp->content_transfer_encoding="8bit";
-
+#endif
 		if (is_flowed_fwd)
 		{
 			struct rfc2646fwd *p=rfc2646fwd_alloc(&write_wrap_func,
@@ -366,8 +468,9 @@ static int mkforward(struct rfc2045_mkreplyinfo *ri)
 			if (!p)
 				return (-1);
 
+#if 0
 			ri->rfc2045partp->content_transfer_encoding=save_te;
-
+#endif
 			parser=RFC2646FWD_PARSEALLOC(p);
 
 			if (!parser)
@@ -393,7 +496,9 @@ static int mkforward(struct rfc2045_mkreplyinfo *ri)
 
 		ri->rfc2045partp->startbody=save_start_body;
 		ri->rfc2045partp->endpos=save_end_pos;
+#if 0
 		ri->rfc2045partp->content_transfer_encoding=save_te;
+#endif
 	}
 
 	if (boundary)
@@ -816,15 +921,18 @@ static int mkreply(struct rfc2045_mkreplyinfo *ri)
 	writes(ri,"Subject: Re: ");
 	if (subject)
 	{
-		int	dummy;
-		char	*s=rfc822_coresubj_nouc(subject, &dummy);
+		char	*s=rfc822_coresubj_keepblobs(subject);
 
 		writes(ri, s ? s:"");
 		if (s)	free(s);
 		free(subject);
 	}
 
-	writes(ri, "\n\n");
+	writes(ri, "\nMime-Version: 1.0\n");
+	writes(ri, "Content-Type: text/plain; charset=\"");
+	writes(ri, ri->charset);
+	writes(ri, "\"\n");
+	writes(ri, "\n");
 	if (whowrote)
 	{
 		writes(ri, whowrote);
